@@ -410,97 +410,6 @@ fn check_group_creation_and_group_code_login() {
                         })
 }
 
-#[test]
-fn check_group_creation_and_group_code_login_no_data() {
-    start_server_and_fn(8093,
-                        |conn| {
-                            addsimpleuser(conn, "testusr".to_string(), "testpw".to_string(), true, false);
-                        },
-                        || {
-                            let client = reqwest::Client::builder().cookie_store(true)
-                                                                   .redirect(reqwest::RedirectPolicy::none())
-                                                                   .build()
-                                                                   .unwrap();
-
-                            let resp = login(8093, &client, "testusr", "testpw");
-                            assert_eq!(resp.status(), StatusCode::FOUND);
-
-                            let mut resp = client.get("http://localhost:8093").send().unwrap();
-                            assert_eq!(resp.status(), StatusCode::OK);
-
-                            let content = resp.text().unwrap();
-                            assert!(content.contains("[Lehrer]"));
-                            assert!(content.contains("Gruppenverwaltung"));
-
-                            let mut resp = client.get("http://localhost:8093/group/").send().unwrap();
-                            assert_eq!(resp.status(), StatusCode::OK);
-
-                            let content = resp.text().unwrap();
-                            assert!(content.contains("Gruppe anlegen"));
-
-                            let params =
-                                [("name", "WrongGroupname"), ("tag", "WrongMarker"), ("csrf_token", "76CfTPJaoz")];
-                            let resp = client.post("http://localhost:8093/group/").form(&params).send().unwrap();
-                            assert_eq!(resp.status(), StatusCode::FORBIDDEN);
-
-                            let pos = content.find("type=\"hidden\" name=\"csrf_token\" value=\"")
-                                             .expect("CSRF-Token not found");
-                            let csrf = &content[pos + 39..pos + 49];
-                            let params = [("name", "Groupname"), ("tag", "Marker"), ("csrf_token", csrf)];
-                            let resp = client.post("http://localhost:8093/group/").form(&params).send().unwrap();
-                            assert_eq!(resp.status(), StatusCode::FOUND);
-
-                            let mut resp = client.get("http://localhost:8093/group/").send().unwrap();
-                            let content = resp.text().unwrap();
-                            assert!(!content.contains("WrongGroupname"));
-
-                            let pos =
-                                content.find("<td><a href=\"/group/1\">Groupname</a></td>").expect("Group not found");
-                            let groupcode = &content[pos + 58..pos + 65];
-
-                            // New client to test group code login
-                            let client = reqwest::Client::builder().cookie_store(true)
-                                                                   .redirect(reqwest::RedirectPolicy::none())
-                                                                   .build()
-                                                                   .unwrap();
-
-                            let resp = login_code(8093, &client, groupcode);
-                            assert_eq!(resp.status(), StatusCode::FOUND);
-
-                            let mut set_cookie = resp.headers().get_all("Set-Cookie").iter();
-                            assert!(set_cookie.next().is_some());
-                            assert!(set_cookie.next().is_none());
-
-                            let location = resp.headers().get(reqwest::header::LOCATION).unwrap().to_str().unwrap();
-                            assert_eq!(location, "http://localhost:8093/profile?status=firstlogin");
-
-                            let mut resp = client.get(location).send().unwrap();
-                            let content = resp.text().unwrap();
-
-                            let pos = content.find("<p>Login-Code: ").expect("Logincode not found");
-                            let logincode = &content[pos + 15..pos + 24];
-
-                            // New client to test login code login
-                            let client = reqwest::Client::builder().cookie_store(true)
-                                                                   .redirect(reqwest::RedirectPolicy::none())
-                                                                   .build()
-                                                                   .unwrap();
-
-                            let resp = login_code(8093, &client, logincode);
-                            assert_eq!(resp.status(), StatusCode::FOUND);
-
-                            let location = resp.headers().get(reqwest::header::LOCATION).unwrap().to_str().unwrap();
-                            assert_eq!(location, "http://localhost:8093/");
-
-                            // Client is forwarded to login page?
-                            let resp = client.get("http://localhost:8093/").send().unwrap();
-                            assert_eq!(resp.status(), StatusCode::FOUND);
-
-                            let location = resp.headers().get(reqwest::header::LOCATION).unwrap().to_str().unwrap();
-                            assert_eq!(location, "http://localhost:8093/profile?status=firstlogin");
-                        })
-}
-
 #[test]
 fn check_contest_start() {
     start_server_and_fn(8085,
@@ -1055,3 +964,219 @@ fn check_contest_requirement() {
                             assert!(content.contains("<a href=\"/task/8\">☆☆☆☆</a></li>"));
                         })
 }
+
+#[test]
+fn check_group_creation_and_group_code_login_no_data() {
+    start_server_and_fn(8093,
+                        |conn| {
+                            addsimpleuser(conn, "testusr".to_string(), "testpw".to_string(), true, false);
+                        },
+                        || {
+                            let client = reqwest::Client::builder().cookie_store(true)
+                                                                   .redirect(reqwest::RedirectPolicy::none())
+                                                                   .build()
+                                                                   .unwrap();
+
+                            let resp = login(8093, &client, "testusr", "testpw");
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let mut resp = client.get("http://localhost:8093").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert!(content.contains("[Lehrer]"));
+                            assert!(content.contains("Gruppenverwaltung"));
+
+                            let mut resp = client.get("http://localhost:8093/group/").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert!(content.contains("Gruppe anlegen"));
+
+                            let params =
+                                [("name", "WrongGroupname"), ("tag", "WrongMarker"), ("csrf_token", "76CfTPJaoz")];
+                            let resp = client.post("http://localhost:8093/group/").form(&params).send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::FORBIDDEN);
+
+                            let pos = content.find("type=\"hidden\" name=\"csrf_token\" value=\"")
+                                             .expect("CSRF-Token not found");
+                            let csrf = &content[pos + 39..pos + 49];
+                            let params = [("name", "Groupname"), ("tag", "Marker"), ("csrf_token", csrf)];
+                            let resp = client.post("http://localhost:8093/group/").form(&params).send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let mut resp = client.get("http://localhost:8093/group/").send().unwrap();
+                            let content = resp.text().unwrap();
+                            assert!(!content.contains("WrongGroupname"));
+
+                            let pos =
+                                content.find("<td><a href=\"/group/1\">Groupname</a></td>").expect("Group not found");
+                            let groupcode = &content[pos + 58..pos + 65];
+
+                            // New client to test group code login
+                            let client = reqwest::Client::builder().cookie_store(true)
+                                                                   .redirect(reqwest::RedirectPolicy::none())
+                                                                   .build()
+                                                                   .unwrap();
+
+                            let resp = login_code(8093, &client, groupcode);
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let mut set_cookie = resp.headers().get_all("Set-Cookie").iter();
+                            assert!(set_cookie.next().is_some());
+                            assert!(set_cookie.next().is_none());
+
+                            let location = resp.headers().get(reqwest::header::LOCATION).unwrap().to_str().unwrap();
+                            assert_eq!(location, "http://localhost:8093/profile?status=firstlogin");
+
+                            let mut resp = client.get(location).send().unwrap();
+                            let content = resp.text().unwrap();
+
+                            let pos = content.find("<p>Login-Code: ").expect("Logincode not found");
+                            let logincode = &content[pos + 15..pos + 24];
+
+                            // New client to test login code login
+                            let client = reqwest::Client::builder().cookie_store(true)
+                                                                   .redirect(reqwest::RedirectPolicy::none())
+                                                                   .build()
+                                                                   .unwrap();
+
+                            let resp = login_code(8093, &client, logincode);
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let location = resp.headers().get(reqwest::header::LOCATION).unwrap().to_str().unwrap();
+                            assert_eq!(location, "http://localhost:8093/");
+
+                            // Client is forwarded to login page?
+                            let resp = client.get("http://localhost:8093/").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let location = resp.headers().get(reqwest::header::LOCATION).unwrap().to_str().unwrap();
+                            assert_eq!(location, "http://localhost:8093/profile?status=firstlogin");
+                        })
+}
+
+#[test]
+fn check_contest_timelimit_student() {
+    start_server_and_fn(8094,
+                        |conn| {
+                            addsimpleuser(conn, "testusr".to_string(), "testpw".to_string(), false, false);
+
+                            let mut now = time::get_time();
+                            now.sec -= 90; // Have the contest started more than a minute ago.
+                            conn.execute(
+                             "INSERT INTO participation (contest, session, start_date)
+                                SELECT $1, id, $2 FROM session WHERE username = 'testusr'",
+                             &[&1, &now]).unwrap();
+                        },
+                        || {
+                            let client = reqwest::Client::builder().cookie_store(true)
+                                                                   .redirect(reqwest::RedirectPolicy::none())
+                                                                   .build()
+                                                                   .unwrap();
+
+                            let resp = login(8094, &client, "testusr", "testpw");
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let resp = client.get("http://localhost:8094/contest/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let resp = client.get("http://localhost:8094/task/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let mut resp = client.get("http://localhost:8094/profile").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            let pos = content.find("type=\"hidden\" name=\"csrf_token\" value=\"")
+                                             .expect("CSRF-Token not found");
+                            let csrf = &content[pos + 39..pos + 49];
+
+                            let mut resp = client.get("http://localhost:8094/load/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert_eq!(content, "{}");
+
+                            let params = [("data", "SomeData"), ("grade", "67"), ("csrf_token", csrf)];
+                            let resp = client.post("http://localhost:8094/save/1").form(&params).send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::UNAUTHORIZED);
+
+                            let mut resp = client.get("http://localhost:8094/load/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert_eq!(content, "{}");
+
+                            let mut resp = client.get("http://localhost:8094/contest/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert!(content.contains("<a href=\"/task/1\">☆☆☆</a></li>"));
+                            assert!(content.contains("<a href=\"/task/2\">☆☆☆☆</a></li>"));
+                        })
+}
+
+#[test]
+fn check_contest_timelimit_teacher() {
+    start_server_and_fn(8095,
+                        |conn| {
+                            addsimpleuser(conn, "testusr".to_string(), "testpw".to_string(), true, false);
+
+                            let mut now = time::get_time();
+                            now.sec -= 90; // Have the contest started more than a minute ago.
+                            conn.execute(
+                             "INSERT INTO participation (contest, session, start_date)
+                                SELECT $1, id, $2 FROM session WHERE username = 'testusr'",
+                             &[&1, &now]).unwrap();
+                        },
+                        || {
+                            let client = reqwest::Client::builder().cookie_store(true)
+                                                                   .redirect(reqwest::RedirectPolicy::none())
+                                                                   .build()
+                                                                   .unwrap();
+
+                            let resp = login(8095, &client, "testusr", "testpw");
+                            assert_eq!(resp.status(), StatusCode::FOUND);
+
+                            let resp = client.get("http://localhost:8095/contest/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let resp = client.get("http://localhost:8095/task/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let mut resp = client.get("http://localhost:8095/profile").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            let pos = content.find("type=\"hidden\" name=\"csrf_token\" value=\"")
+                                             .expect("CSRF-Token not found");
+                            let csrf = &content[pos + 39..pos + 49];
+
+                            let mut resp = client.get("http://localhost:8095/load/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert_eq!(content, "{}");
+
+                            let params = [("data", "SomeData"), ("grade", "67"), ("csrf_token", csrf)];
+                            let mut resp = client.post("http://localhost:8095/save/1").form(&params).send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert_eq!(content, "{}");
+
+                            let mut resp = client.get("http://localhost:8095/load/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert_eq!(content, "SomeData");
+
+                            let mut resp = client.get("http://localhost:8095/contest/1").send().unwrap();
+                            assert_eq!(resp.status(), StatusCode::OK);
+
+                            let content = resp.text().unwrap();
+                            assert!(content.contains("<a href=\"/task/1\">★★☆</a></li>"));
+                            assert!(content.contains("<a href=\"/task/2\">☆☆☆☆</a></li>"));
+                        })
+}
